

-module(fac).
-behaviour(gen_server).

-export([init/1, handle_call/3, handle_cast/2,
      handle_info/2, terminate/2, code_change/3]).
-export([start/1, stop/1]).
-export([fac/4]).
-export([run/1]).
-export([handler/2]).

-record(server, {
       self,
       port,
       players,
       pending
      }).

new(Port) ->
     #server {
      self = self(),
      port = Port
     }.

run(N) ->
%%    start(2001),
    fac("localhost",2001,20,N).

start(Port) ->
     gen_server:start(fac, [Port], []).

init([Port]) ->
     process_flag(trap_exit, true),
     Server = new(Port),
     tcp_server:start_raw_server(Port,
				 fun(Msg, Sock) -> handler(Msg, Sock) end,
				 2048),
     {ok, Server}.

stop(Server) ->
     gen_server:cast(Server, stop).

terminate(normal, Server) ->
     tcp_server:stop(Server#server.port),
     ok.

handle_cast(stop, Server) ->
     {stop, normal, Server};

handle_cast(Event, Server) ->
     error_logger:info_report([{module, ?MODULE},
                   {line, ?LINE},
                   {self, self()},
                   {message, Event}]),
     {noreply, Server}.

handle_call(Event, From, Server) ->
     error_logger:info_report([{module, ?MODULE},
                   {line, ?LINE},
                   {self, self()},
                   {message, Event},
                   {from, From}]),
     {noreply, Server}.

handle_info({'EXIT', _Pid, _Reason}, Server) ->
     %% child exit?
     {noreply, Server};

handle_info(Info, Server) ->
     error_logger:info_report([{module, ?MODULE},
                   {line, ?LINE},
                   {self, self()},
                   {message, Info}]),
     {noreply, Server}.

code_change(_OldVsn, Server, _Extra) ->
     {ok, Server}.

%%%
%%% Implementation
%%%

fac(N) when N > 0 ->
      N * fac(N - 1);
fac(0) ->
     1.

fac(_Host, _Port, _N, 0) ->
     ok;

fac(Host, Port, N, Max) ->
    Fun = fun() ->
		  case tcp_server:start_client(Host, Port, 2048) of
		      {ok, Sock} ->
			  case gen_tcp:send(Sock, <<Max:32, N:32>>) of
			      ok ->
				  receive
				      {tcp, Sock, <<Fac:2048>>} ->
%%%                        io:format("#~w: fac(~w) = ~w~n",
%%%                              [Max, N, Fac]);
					  ok;
				      {error, closed} ->
					  io:format("#~w: connection closed~n",
						    [Max]);
				      Any ->
					  io:format("#~w: receive: ~w~n",
						    [Max, Any])
				  end,
				  gen_tcp:close(Sock);
			      Any ->
				  io:format("#~w: gen_tcp: ~w~n", [Max, Any])
			  end;
		      Any ->
			  io:format("#~w: start_client: ~w~n", [Max, Any])
		  end
	  end,
    spawn(Fun),
    fac(Host, Port, N, Max - 1).

handler(<<Iter:32, N:32>>, Socket) ->
    io:format("#~w: Calculating factorial(~w)~n", [Iter, N]),
    Fac = fac(N),
    gen_tcp:send(Socket, <<Fac:2048>>).
