
-module(tcp_server).

-export([server/0]).
-export([start_client/3]).
-export([raw_listen/3]).


server() ->
    ets:new(stats, [set, public, named_table]),
    raw_listen(2001,2048,fun(M,S) ->
				 fac:handler(M,S)
			 end).

start_client(Host, Port, Length) ->
      gen_tcp:connect(Host, Port,
              [binary,
               {active, true},
               {packet, 2},
               {packet_size, Length}], 60000).



raw_listen(Port, Length, Fun) ->
    T = ets:new(counter, [set, public]),
    ets:insert(T, {total, 0}),
    Options = [binary,
	       {nodelay,true},
	       {packet_size, Length},
	       {packet, 2},
	       {reuseaddr, true},
	       {active, true}],
    case inet_tcp:listen(Port, Options) of
	{ok, S} ->
	    start_acceptor(S, Fun, T);
	Error ->
	    erlang:fault(Error)
    end.

start_acceptor(S, Fun, T) ->
    Parent = self(),
    spawn_opt(
      fun() ->
	      case timer:tc(inet_tcp, accept, [S]) of
		  {Time, {ok, Sock}} ->
		      ets:insert(stats, {self(), Time}),
		      Parent ! accepted,
		      process_flag(priority, normal),
		      erlang:yield(),
		      receive
			  {tcp, Sock, Msg} ->
			      Prev = ets:update_counter(T, total, 1),
			      io:format("<~p> ", [Prev]),
			      Fun(Msg, Sock),
			      inet_tcp:close(Sock);
			  Msg ->
			      io:format("Msg = ~p~n", [Msg])
		      end;
		  Error ->
		      erlang:fault(Error)
	      end
      end, [link, {priority, high}]),
    receive
	accepted ->
	    start_acceptor(S, Fun, T)
    end.



